#ifndef GRANULAR_GRAIN_
#define GRANULAR_GRAIN_

#include <stdio.h>
#include "contact.h"

#define MAXCONTACTS 32

struct grain {

	double diameter;
	double pos[3];
	double vel[3];
	double acc[3];
	int acc_lock[3];
	double force[3];
	double angpos[3];
	double angvel[3];
	double angacc[3];
	double torque[3];
	double disp[3];
	double forceext[3];
	double density;
	int fixed;
	int rotating;
	int enabled;
	double youngs_modulus;
	double poissons_ratio;
	double friction_coeff;
	double damping_n;
	double damping_t;
	double tensile_strength;
	double shear_strength;
	double fracture_toughness;
	size_t gridpos[3];
	size_t ncontacts;
	struct contact contacts[MAXCONTACTS];
	double contact_stress[3];
	double thermal_energy;
	int color;
};

struct grain grain_new(void);
void grain_defaults(struct grain *g);

void grain_print(FILE *stream, const struct grain *g);
struct grain * grain_read(char *line);

int grain_check_values(const struct grain *g);

double grain_volume(const struct grain *g);
double grain_mass(const struct grain *g);
double grain_moment_of_inertia(const struct grain *g);
void grain_zero_kinematics(struct grain *g);
void grain_force_reset(struct grain *g);
double grain_translational_kinetic_energy(const struct grain *g);
double grain_rotational_kinetic_energy(const struct grain *g);
double grain_kinetic_energy(const struct grain *g);
double grain_thermal_energy(const struct grain *g);
void grain_temporal_integration(struct grain *g, double dt);

void grain_register_contact(struct grain *g, size_t i, size_t j,
                            double centerdist[3], double overlap);

double grain_stiffness_normal(const struct grain *g);
double grain_stiffness_tangential(const struct grain *g);
double grain_collision_time(const struct grain *g);

#endif
