#include <stdio.h>
#include <math.h>
#include "grain.h"
#include "arrays.h"

#define VTK_FLOAT_FMT "%.17g "

#define VTK_XML_SCALAR(M, N, T, F) \
	fprintf(stream,\
	        "\t\t\t\t<DataArray type=\"" T "\" Name=\"" N "\" "\
	        "NumberOfComponents=\"1\" format=\"ascii\">\n");\
	for (i = 0; i < ng; i++)\
		fprintf(stream, F, grains[i].M);\
	fprintf(stream, "\n\t\t\t\t</DataArray>\n");

#define VTK_XML_VECTOR(M, N, T, F) \
	fprintf(stream,\
	        "\t\t\t\t<DataArray type=\"" T "\" Name=\"" N "\" "\
	        "NumberOfComponents=\"3\" format=\"ascii\">\n");\
	for (i = 0; i < ng; i++)\
		for (d = 0; d < 3; d++)\
			fprintf(stream, F, grains[i].M[d]);\
	fprintf(stream, "\n\t\t\t\t</DataArray>\n");

void
grains_print(FILE *stream, const struct grain *grains, size_t ng)
{
	size_t i;

	for (i = 0; i < ng; i++)
		grain_print(stream, &grains[i]);
}

void
grains_print_vtk(FILE *stream, const struct grain *grains, size_t ng)
{
	size_t i, d;

	fprintf(stream,
	        "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
	        "<VTKFile type=\"UnstructuredGrid\" version=\"1.0\" "
	        "byte_order=\"LittleEndian\">\n"
	        "\t<UnstructuredGrid>\n"
	        "\t\t<Piece NumberOfPoints=\"%zu\" NumberOfCells=\"0\">\n", ng);
	fprintf(stream, "\t\t\t<Points>\n");
	VTK_XML_VECTOR(pos, "Position [m]", "Float64", VTK_FLOAT_FMT);
	fprintf(stream, "\t\t\t</Points>\n");

	fprintf(stream,
	        "\t\t\t<Cells>\n"
	        "\t\t\t\t<DataArray type=\"Int32\" Name=\"connectivity\" "
	        "NumberOfComponents=\"1\" format=\"ascii\"/>\n"
	        "\t\t\t\t<DataArray type=\"Int32\" Name=\"offsets\" "
	        "NumberOfComponents=\"1\" format=\"ascii\"/>\n"
	        "\t\t\t\t<DataArray type=\"UInt8\" Name=\"types\" "
	        "NumberOfComponents=\"1\" format=\"ascii\"/>\n"
	        "\t\t\t</Cells>\n");

	fprintf(stream,
	        "\t\t\t<PointData Scalars=\"Diameter [m]\" "
	        "Vectors=\"Angular position [-]\">\n");

	VTK_XML_SCALAR(diameter, "Diameter [m]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_VECTOR(vel, "Velocity [m/s]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_VECTOR(acc, "Acceleration [m/s^2]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_VECTOR(acc_lock, "Acceleration lock [-]", "Int64", "%d ");
	VTK_XML_VECTOR(force, "Force [N]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_VECTOR(angpos, "Angular position [rad]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_VECTOR(angvel, "Angular velocity [rad/s]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_VECTOR(angacc, "Angular acceleration [rad/s^2]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_VECTOR(torque, "Torque [N/m]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_VECTOR(disp, "Displacement [m]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_VECTOR(forceext, "External body force [N]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_SCALAR(density, "Density [kg/m^3]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_SCALAR(fixed, "Fixed [-]", "Int64", "%d ");
	VTK_XML_SCALAR(rotating, "Rotating [-]", "Int64", "%d ");
	VTK_XML_SCALAR(enabled, "Enabled [-]", "Int64", "%d ");
	VTK_XML_SCALAR(youngs_modulus, "Young's modulus [Pa]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_SCALAR(poissons_ratio, "Poisson's ratio [-]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_SCALAR(friction_coeff, "Friction coefficient [-]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_SCALAR(damping_n, "Contact-normal damping constant [1/s]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_SCALAR(damping_t, "Contact-tangential damping constant [1/s]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_SCALAR(tensile_strength, "Tensile strength [Pa]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_SCALAR(shear_strength, "Shear strength [Pa]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_SCALAR(fracture_toughness, "Fracture toughness [Pa]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_VECTOR(gridpos, "Grid position [-]", "UInt64", "%zu ");
	VTK_XML_SCALAR(ncontacts, "Number of contacts [-]", "UInt64", "%zu ");
	VTK_XML_VECTOR(contact_stress, "Contact stress [Pa]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_SCALAR(thermal_energy, "Thermal energy [J]", "Float64", VTK_FLOAT_FMT);
	VTK_XML_SCALAR(color, "Color [-]", "Int64", "%d ");

	fprintf(stream, "\t\t\t</PointData>\n");

	fprintf(stream,
	        "\t\t</Piece>\n"
	        "\t</UnstructuredGrid>\n"
	        "</VTKFile>\n");
}

void
grains_print_energy(FILE *stream, const struct grain *grains, size_t ng)
{
	size_t i;
	double E_kin = 0.0,
	       E_rot = 0.0,
	       E_thermal = 0.0;
	
	for (i = 0; i < ng; i++) {
		E_kin += grain_translational_kinetic_energy(&grains[i]);
		E_rot += grain_rotational_kinetic_energy(&grains[i]);
		E_thermal += grain_thermal_energy(&grains[i]);
	}

	fprintf(stream, "%.17g\t%.17g\t%.17g\t%.17g\n",
	        E_kin + E_rot + E_thermal,
	        E_kin, E_rot, E_thermal);
}

double
grains_minimum_grain_edge(const struct grain *grains, size_t ng, int d)
{
	size_t i;
	double res = INFINITY;

	for (i = 0; i < ng; i++)
		if (res > grains[i].pos[d] - grains[i].diameter / 2.0)
			res = grains[i].pos[d] - grains[i].diameter / 2.0;

	return res;
}

double
grains_maximum_grain_edge(const struct grain *grains, size_t ng, int d)
{
	size_t i;
	double res = -INFINITY;

	for (i = 0; i < ng; i++)
		if (res < grains[i].pos[d] + grains[i].diameter / 2.0)
			res = grains[i].pos[d] + grains[i].diameter / 2.0;

	return res;
}

/* Hertz-Mindlin contact model for compressible spheres */
void
grains_interact(struct grain *g_i, struct grain *g_j, int ic, double dt)
{
	int d;
	double f_n_ij[3], f_t_ij[3],
		   v_ij[3], v_n_ij[3], v_t_ij[3], v_n_dot_n_ij,
	       delta_ij,
		   n_ij[3],
		   d_i, d_j,
		   r_ij[3], r_ij_norm,
	       m_i, m_j, m_ij,
		   mu_ij,
		   gamma_n_ij, gamma_t_ij,
	       k_n_ij, k_t_ij, A_ij,
		   angvel_ij[3], u_t_dot_v_ij,
		   *angvel_cross_r_ij, *f_t_cross_r_ij;

	delta_ij = g_i->contacts[ic].overlap;

	if (delta_ij < 0.0) {  /* TODO: implement tensile strength */
		g_i->contacts[ic].active = 0;
		g_i->ncontacts--;
		return;
	}

	d_i = g_i->diameter;
	d_j = g_j->diameter;
	m_i = grain_mass(g_i);
	m_j = grain_mass(g_j);
	m_ij = m_i * m_j / (m_i + m_j);
	r_ij_norm = euclidean_norm(g_i->contacts[ic].centerdist, 3);

	for (d = 0; d < 3; d++) {
		r_ij[d] = g_i->contacts[ic].centerdist[d];
		n_ij[d] = r_ij[d]/r_ij_norm;
		v_ij[d] = g_i->vel[d] - g_j->vel[d];
		angvel_ij[d] = g_i->angvel[d] + g_j->angvel[d];
	}

	v_n_dot_n_ij = dot(v_ij, n_ij, 3);
	angvel_cross_r_ij = cross(angvel_ij, r_ij);

	for (d = 0; d < 3; d++) {
		v_n_ij[d] = v_n_dot_n_ij * n_ij[d];
		v_t_ij[d] = v_ij[d] - v_n_ij[d] - 0.5 * angvel_cross_r_ij[d];
	}

	mu_ij = fmin(g_i->friction_coeff, g_j->friction_coeff);
	gamma_n_ij = fmin(g_i->damping_n, g_j->damping_n);
	gamma_t_ij = fmin(g_i->damping_t, g_j->damping_t);

	k_n_ij = fmin(grain_stiffness_normal(g_i),
	              grain_stiffness_normal(g_j));
	k_t_ij = fmin(grain_stiffness_tangential(g_i),
	              grain_stiffness_tangential(g_j));

	A_ij = sqrt(delta_ij) * sqrt(d_i * d_j / (2.0 * (d_i + d_j)));

	for (d = 0; d < 3; d++) {
		f_n_ij[d] = A_ij * (k_n_ij * delta_ij * n_ij[d] 
							- m_ij * gamma_n_ij * v_n_ij[d]);
		f_t_ij[d] = A_ij * (-k_t_ij * g_i->contacts[ic].tandisp[d]
		                    - m_ij * gamma_t_ij * v_t_ij[d]);
	}

	if (euclidean_norm(f_t_ij, 3) >= euclidean_norm(f_n_ij, 3) * mu_ij)
		return; /* TODO: limit f_t_ij according to Coulomb criterion */

	f_t_cross_r_ij = cross(f_t_ij, r_ij);

	for (d = 0; d < 3; d++) {
		g_i->force[d] += f_n_ij[d] + f_t_ij[d];
		g_j->force[d] -= f_n_ij[d] + f_t_ij[d];
		g_i->torque[d] += -0.5 * f_t_cross_r_ij[d];
		g_j->torque[d] -= -0.5 * f_t_cross_r_ij[d];
	}

	g_i->contacts[ic].age += dt;
	u_t_dot_v_ij = dot(g_i->contacts[ic].tandisp, v_ij, 3);
	for (d = 0; d < 3; d++)
		g_i->contacts[ic].tandisp[d] += (v_t_ij[d]
		                              - u_t_dot_v_ij * r_ij[d] 
									  / (r_ij_norm * r_ij_norm))
									  * dt;

	free(angvel_cross_r_ij);
	free(f_t_cross_r_ij);
}
