#include <stdlib.h>
#include <err.h>
#include <unistd.h>
#include "granular.h"
#include "arg.h"

char *argv0;

static void
usage(void)
{
	errx(1, "usage: %s "
		 "[-e end-time] "
		 "[-g gravity-accel] "
		 "[-h] "
		 "[-I file-interval] "
		 "[-j time-step] "
		 "[-t curr-time] "
		 "[name]", argv0);
}

int
main(int argc, char *argv[])
{
	int ret;
	struct simulation sim = sim_new();
	double grav = 0.0;

#ifdef __OpenBSD__
	if (pledge("stdio wpath cpath", NULL) == -1)
		err(2, "pledge");
#endif

	sim_defaults(&sim);

	ARGBEGIN {
	case 'e':
		sim.t_end = atof(EARGF(usage()));
		break;
	case 'g':
		grav = atof(EARGF(usage()));
		break;
	case 'h':
		usage();
		break;
	case 'I':
		sim.dt_file = atof(EARGF(usage()));
		break;
	case 'j':
		sim.dt = atof(EARGF(usage()));
		break;
	case 't':
		sim.t = atof(EARGF(usage()));
		break;
	default:
		usage();
	} ARGEND;

	if (argc == 1 && argv[0]) {
		ret = snprintf(sim.name, sizeof(sim.name), "%s", argv[0]);
		if (ret < 0 || (size_t)ret >= sizeof(sim.name))
			errx(1, "%s: sim.name snprintf", __func__);
	} else if (argc > 1)
		usage();

	sim_read_grains(&sim, stdin);
	sim_add_acceleration_scalar(&sim, grav, 1);
	sim_set_timestep(&sim);
	if (sim.t < sim.t_end)
		sim_run_time_loop(&sim);
	sim_print_grains(&sim, stdout);
	sim_free(&sim);

	return 0;
}
