#include <stdlib.h>
#include <err.h>
#include <unistd.h>
#include "granular.h"
#include "arg.h"

char *argv0;

static void
usage(void)
{
	errx(1, "usage: %s "
		 "[-d diameter] "
		 "[-f] "
		 "[-h] "
		 "[-R] "
		 "[-u vx] "
		 "[-v vy] "
		 "[-w vz] "
		 "[-x x] "
		 "[-y y] "
		 "[-z z]", argv0);
}

int
main(int argc, char *argv[])
{
	struct grain g = grain_new();

#ifdef __OpenBSD__
	if (pledge("stdio", NULL) == -1)
		err(2, "pledge");
#endif

	ARGBEGIN {
	case 'd':
		g.diameter = atof(EARGF(usage()));
		break;
	case 'f':
		g.fixed = 1;
		break;
	case 'h':
		usage();
		break;
	case 'R':
		g.rotating = 0;
		break;
	case 'u':
		g.vel[0] = atof(EARGF(usage()));
		break;
	case 'v':
		g.vel[1] = atof(EARGF(usage()));
		break;
	case 'w':
		g.vel[2] = atof(EARGF(usage()));
		break;
	case 'x':
		g.pos[0] = atof(EARGF(usage()));
		break;
	case 'y':
		g.pos[1] = atof(EARGF(usage()));
		break;
	case 'z':
		g.pos[2] = atof(EARGF(usage()));
		break;
	default:
		usage();
	} ARGEND;

	if (argc > 1)
		usage();

	grain_print(stdout, &g);

	return 0;
}
