#include <stdlib.h>
#include <err.h>
#include <unistd.h>
#include "granular.h"
#include "arg.h"

char *argv0;

static void
usage(void)
{
	errx(1, "usage: %s "
		 "[-D max-diameter] "
		 "[-d min-diameter] "
		 "[-hP] "
		 "[-p padding-factor] "
		 "[-t] "
		 "[-X x-offset] "
		 "[-x nx] "
		 "[-Y y-offset] "
		 "[-y ny] "
		 "[-Z z-offset] "
		 "[-z nz]", argv0);
}

int
main(int argc, char *argv[])
{
	size_t n[] = {10, 10, 1};
	double origo[] = {0.0, 0.0, 0.0};
	double padding = 0.0;
	double d_max = 1.0, d_min = 1.0;
	struct simulation sim = sim_new();
	int packing = 0;
	double (*sizefunc)(double min, double max) = random_value_uniform;

#ifdef __OpenBSD__
	if (pledge("stdio", NULL) == -1)
		err(2, "pledge")
#endif

	ARGBEGIN {
	case 'D':
		d_max = atof(EARGF(usage()));
		break;
	case 'd':
		d_min = atof(EARGF(usage()));
		break;
	case 'h':
		usage();
		break;
	case 'P':
		sizefunc = (*random_value_powerlaw);
		break;
	case 'p':
		padding = atof(EARGF(usage()));
		break;
	case 't':
		packing = 1;
		break;
	case 'X':
		origo[0] = atof(EARGF(usage()));
		break;
	case 'x':
		n[0] = atoi(EARGF(usage()));
		break;
	case 'Y':
		origo[1] = atof(EARGF(usage()));
		break;
	case 'y':
		n[1] = atoi(EARGF(usage()));
		break;
	case 'Z':
		origo[2] = atof(EARGF(usage()));
		break;
	case 'z':
		n[2] = atoi(EARGF(usage()));
		break;
	default:
		usage();
	} ARGEND;

	if (argc > 1)
		usage();
	
	switch (packing) {
	case 0:
		sim.ng += rectangular_packing(&sim.grains, n,
		                              d_min / 2.0, d_max / 2.0,
		                              sizefunc, padding, origo);
		break;
	case 1:
		sim.ng += triangular_packing(&sim.grains, n,
		                             d_min / 2.0, d_max / 2.0,
		                             sizefunc, padding, origo);
		break;
	default:
		errx(1, "unknown packing mode");
	}

	sim_print_grains(&sim, stdout);
	sim_free(&sim);

	return 0;
}
