#!/bin/sh

# stop the shell script if there's an error
set -e

# set a common id string for files
id=many-grain-collision

# remove any old files associated with this id
rm -f ${id}.grains.*.{tsv,png} ${id}.mp4

# create a single grain and a granular packing, and simulate the interaction over time
(granulargrain -R -u 1.0 -y 2.5; granularpacking -X 2.2) | granular -e 4.0 -I 0.1 ${id}

# plot grain positions over time, colored by their horizontal velocity
for f in ${id}.grains.*.tsv; do
	granular2img -f '$5' -l 'x velocity [m/s]' -t png < "$f" > "${f%.tsv}.png"
done

# combine grain plots into animation
ffmpeg -y -framerate 5 -i ${id}.grains.%05d.png \
	-c:v libx264 -r 30 -pix_fmt yuv420p ${id}.mp4

# open animation
xdg-open ${id}.mp4 
