#include <stdio.h>
#include <stdlib.h>
#include <err.h>
#include <math.h>
#include "util.h"
#include "granular.h"
#include "grain.h"
#include "arrays.h"

size_t
rectangular_packing(struct grain **grains,
                    size_t n[3],
                    double diameter_min, double diameter_max,
                    double (*gsd)(double min, double max),
                    double padding_factor,
                    double origo[3])
{
	size_t i, j, k, l, ng, N = 0;
	double dx_padding = diameter_max * padding_factor;
	double dx = diameter_max + dx_padding;

	if (diameter_max < diameter_min)
		errx(1, "%s: diameter_max (%g) is smaller than diameter_min (%g)",
		     __func__, diameter_max, diameter_min);

	if (!(*grains = calloc(n[0] * n[1] * n[2], sizeof(struct grain))))
		err(1, "%s: grains calloc", __func__);

	for (k = 0; k < n[2]; k++)
		for (j = 0; j < n[1]; j++)
			for (i = 0; i < n[0]; i++) {
				N++;
				ng = idx3(i, j, k, n[0], n[1]);
				grain_defaults(&(*grains)[ng]);
				(*grains)[ng].diameter = gsd(diameter_min, diameter_max);
				(*grains)[ng].pos[0] = i * dx + 0.5 * dx + origo[0];
				(*grains)[ng].pos[1] = j * dx + 0.5 * dx + origo[1];
				(*grains)[ng].pos[2] = k * dx + 0.5 * dx + origo[2];
				for (l = 0; l < 3; l++)
					(*grains)[ng].pos[l] +=
						random_value_uniform(-0.5 * dx_padding,
						                     0.5 * dx_padding);
#ifdef VERBOSE
				printf("added grain %zu\n", ng);
#endif
			}

	return N;
}

size_t
triangular_packing(struct grain **grains,
                   size_t n[3],
                   double diameter_min, double diameter_max,
                   double (*gsd)(double min, double max),
                   double padding_factor,
                   double origo[3])
{
	size_t i, j, k, l, ng, N = 0;
	double dx_padding = diameter_max * padding_factor;
	double dx = diameter_max + dx_padding;
	double dy = dx * sin(PI/3.0);

	if (diameter_max < diameter_min)
		errx(1, "%s: diameter_max (%g) is smaller than diameter_min (%g)",
		     __func__, diameter_max, diameter_min);

	if (!(*grains = calloc(n[0] * n[1] * n[2], sizeof(struct grain))))
		err(1, "%s: grains calloc", __func__);

	for (k = 0; k < n[2]; k++)
		for (j = 0; j < n[1]; j++)
			for (i = 0; i < n[0]; i++) {
				N++;
				ng = idx3(i, j, k, n[0], n[1]);
				grain_defaults(&(*grains)[ng]);
				(*grains)[ng].diameter = gsd(diameter_min, diameter_max);
				(*grains)[ng].pos[0] = i * dx + 0.5 * dx + origo[0];
				if (j % 2 == 0)
					(*grains)[ng].pos[0] += 0.5 * dx;
				(*grains)[ng].pos[1] = j * dy + 0.5 * dx + origo[1];
				(*grains)[ng].pos[2] = k * dx + 0.5 * dx + origo[2];
				for (l = 0; l < 2; l++)
					(*grains)[ng].pos[l] +=
						random_value_uniform(-0.5 * dx_padding,
											 0.5 * dx_padding);
#ifdef VERBOSE
				printf("added grain %zu\n", ng);
#endif
			}

	return N;
}
