#ifndef GRANULAR_SIMULATION_
#define GRANULAR_SIMULATION_

#define DEFAULT_SIMULATION_NAME "unnamed"

#include <stdio.h>
#include "grain.h"

/* Simulation settings */
struct simulation {

	/* simulation name to use for output files */
	char name[255];

	/* grain sorting grid */
	size_t nd[3];
	double origo[3];
	double L[3];

	/* temporal state [s] */
	double t;
	double t_end;
	double dt;
	double dt_file;
	size_t n_file;
	size_t iter;
	double t_file;

	/* grains */
	size_t ng;
	struct grain *grains;
};

struct simulation sim_new(void);
void sim_defaults(struct simulation *sim);
void sim_free(struct simulation *sim);

void sim_add_grain(struct simulation *sim, struct grain *g);
void sim_read_grains(struct simulation *sim, FILE *stream);

void sim_print_grains(const struct simulation *sim, FILE *stream);
void sim_print_grains_vtk(const struct simulation *sim, FILE *stream);
void sim_print_grains_energy(const struct simulation *sim, FILE *stream);
void sim_write_output_files(struct simulation *sim);

void sim_force_reset(struct simulation *sim);
void sim_add_acceleration_scalar(struct simulation *sim, double acc, int dimension);
void sim_set_timestep(struct simulation *sim);
void sim_detect_contacts(struct simulation *sim);
void sim_step_time(struct simulation *sim);
void sim_run_time_loop(struct simulation *sim);

#endif
