#!/bin/sh

# stop the shell script if there's an error
set -e

# set a common id string for files
id=two-grain-collision

# remove any old files associated with this id
rm -f ${id}.grains.*.{tsv,png} ${id}.mp4

# create two grains, and simulate their interaction over time
(granulargrain -R -u 0.1; granulargrain -f -x 1.2) | granular -e 4.0 -I 0.1 ${id}

# plot grain positions over time, colored by force magnitude
for f in ${id}.grains.*.tsv; do
	#granular2img -f '$50' -l 'number of contacts' -t png < "$f" > "${f%.tsv}.png"
	#granular2img -f '$5' -l 'x velocity [m/s]' -t png < "$f" > "${f%.tsv}.png"
	granular2img -f '$14' -l 'force_x [N]' -t png < "$f" > "${f%.tsv}.png"
done

# combine grain plots into animation
ffmpeg -y -framerate 5 -i ${id}.grains.%05d.png \
	-c:v libx264 -r 30 -pix_fmt yuv420p ${id}.mp4

# open animation
xdg-open ${id}.mp4 

# calculate bulk energy for each output file
> "${id}.energy.tsv"
for f in ${id}.grains.*.tsv; do
	granularenergy < "$f" >> "${id}.energy.tsv"
done

# plot energy over time
gnuplot -e "set term png;\
            set xlabel 'time step';\
            set ylabel 'Energy [J]';\
			plot '-' u 0:1 w lp t 'Total energy'" \
			 < "${id}.energy.tsv" > "${id}.energy.png"

# open energy plot
xdg-open "${id}.energy.png"
