#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <err.h>
#include "granular.h"

void
warn_parameter_value(const char message[],
                     const double value,
                     int *status)
{
	fprintf(stderr,
			"%s: %s (%.17g)\n",
			__func__, message, value);
	*status = 1;
}

void
check_float(const char name[], const double value, int *status)
{
	int ret;
	char message[100];

	if (isnan(value)) {
		ret = snprintf(message, sizeof(message), "%s is NaN", name);
		if (ret < 0 || (size_t)ret >= sizeof(message))
			err(1, "%s: message parsing", __func__);
		warn_parameter_value(message, value, status);
		*status = 1;
	} else if (isinf(value)) {
		ret = snprintf(message, sizeof(message), "%s is infinite", name);
		if (ret < 0 || (size_t)ret >= sizeof(message))
			err(1, "%s: message parsing", __func__);
		warn_parameter_value(message, value, status);
		*status = 1;
	}
}

void
check_float_non_negative(const char name[], const double value, int *status)
{
	int ret;
	char message[100];

	check_float(name, value, status);
	if (value < 0.0) {
		ret = snprintf(message, sizeof(message), "%s is negative", name);
		if (ret < 0 || (size_t)ret >= sizeof(message))
			err(1, "%s: message parsing", __func__);
		warn_parameter_value(message, value, status);
		*status = 1;
	}
}

void
check_float_positive(const char name[], const double value, int *status)
{
	int ret;
	char message[100];

	check_float(name, value, status);
	if (value <= 0.0) {
		ret = snprintf(message, sizeof(message), "%s is not positive", name);
		if (ret < 0 || (size_t)ret >= sizeof(message))
			err(1, "%s: message parsing", __func__);
		warn_parameter_value(message, value, status);
		*status = 1;
	}
}

void
check_int_bool(const char name[], const int value, int *status)
{
	int ret;
	char message[100];

	if (value < 0 || value > 1) {
		ret = snprintf(message, sizeof(message), "%s is not 0 or 1", name);
		if (ret < 0 || (size_t)ret >= sizeof(message))
			err(1, "%s: message parsing", __func__);
		warn_parameter_value(message, (double)value, status);
		*status = 1;
	}
}

void
check_int_non_negative(const char name[], const int value, int *status)
{	
	int ret;
	char message[100];

	if (value < 0) {
		ret = snprintf(message, sizeof(message), "%s is negative", name);
		if (ret < 0 || (size_t)ret >= sizeof(message))
			err(1, "%s: message parsing", __func__);
		warn_parameter_value(message, (double)value, status);
		*status = 1;
	}
}

double
residual(const double new_val, const double old_val)
{
	return (new_val - old_val) / (old_val + 1e-16);
}

double
randomval(void)
{
	return (double)rand() / RAND_MAX;
}

double
random_value_uniform(double min, double max)
{
	return randomval() * (max - min) + min;
}

double
random_value_powerlaw(double min, double max)
{
	double power = -1.8;
	return pow((pow(max, power + 1.0) - pow(min, power + 1.0)) * randomval()
		+ pow(min, power + 1.0), 1.0 / (power + 1.0));
}

void *
xreallocarray(void *m, size_t n, size_t s)
{
	void *nm;

	if (n == 0 || s == 0) {
		free(m);
		return NULL;
	}
	if (s && n > (size_t) - 1 / s)
		err(1, "realloc: overflow");
	if (!(nm = realloc(m, n * s)))
		err(1, "realloc");

	return nm;
}

double
harmonic_mean(double a, double b)
{
	if (a < 1e-16 || b < 1e-16)
		return 0.0;
	else
		return 2.0 * a * b / (a + b);
}
